#include <stdio.h>
#include <string.h>
#include <time.h>
#include "hdfi.h"
#include "cris.h"
#include "sis.h"
#include "utilities.h"
#include "crisSisGetUtilities.h"

/*

getWorkingStartTime is needed in order to determine if the secondsSince1996
falls within the time that the last non-zeroed struct time stamped
prior to secondsSince1996 was valid.  in other words, search backward from
secondsSince1996 to find the first struct containing real data and determine
if secondsSince1996 falls within the time that struct was valid.

usually this is just a case of going back a second less than one cycle
and seeing if any structs are time stamped between 
(secondsSince1996 - period + 1) and secondsSince1996.  if so, this is
the struct valid at secondsSince1996, and we want to make sure we
include it in any set of structs we are getting.

unfortunately, there are also some non-periodic structs which have
both a start and stop time in the struct.  in these cases we have to
back up to the last non-zero struct and check the stop time to see
if it was valid at secondsSince1996.

*/

/* local subroutine */
uint32 getStartForStructWithStopTime(uint32 secondsSince1996, char *structName);


uint32 getWorkingStartTime(uint32 secondsSince1996, char *structName) {
    if (strstr(structName, "Cris") != NULL) {
	if (strcmp(structName, "BrowseCris") == 0)
	    return secondsSince1996 - CRIS_BROWSE_SAMPLE_PERIOD + 1;
	else if (strcmp(structName, "L1CrisCommandEcho") == 0)
	    return secondsSince1996 - CRIS_COMMAND_ECHO_SAMPLE_PERIOD + 1;
	else if (strcmp(structName, "L1CrisCommandTable") == 0) return
		getStartForStructWithStopTime(secondsSince1996, structName);
	else if (strcmp(structName, "L1CrisDiagnosticEvent") == 0) return
		getStartForStructWithStopTime(secondsSince1996, structName);
	else if (strcmp(structName, "L1CrisEventCycle") == 0)
	    return secondsSince1996 - CRIS_EVENT_CYCLE_SAMPLE_PERIOD + 1;
	else if (strcmp(structName, "L1CrisHighPriorityRate") == 0)
	    return secondsSince1996 - CRIS_HIGH_PRIORITY_RATE_SAMPLE_PERIOD + 1;
	else if (strcmp(structName, "L1CrisHskp") == 0)
	    return secondsSince1996 - CRIS_HOUSEKEEPING_SAMPLE_PERIOD + 1;
	else if (strcmp(structName, "L1CrisLowPriorityRate") == 0)
	    return secondsSince1996 - CRIS_LOW_PRIORITY_RATE_SAMPLE_PERIOD + 1;
	else if (strcmp(structName, "L1CrisSubset") == 0)
	    return secondsSince1996 - CRIS_SUBSET_SAMPLE_PERIOD + 1;
	else if (strcmp(structName, "L1CrisSummary") == 0)
	    return secondsSince1996 - CRIS_SUMMARY_SAMPLE_PERIOD + 1;
    }
    else {
	if (strcmp(structName, "BrowseSis") == 0)
	    return secondsSince1996 - SIS_BROWSE_SAMPLE_PERIOD + 1;
	else if (strcmp(structName, "L1SisCommandEcho") == 0)
	    return secondsSince1996 - SIS_COMMAND_ECHO_SAMPLE_PERIOD + 1;
	else if (strcmp(structName, "L1SisCommandTable") == 0) return
		getStartForStructWithStopTime(secondsSince1996, structName);
	else if (strcmp(structName, "L1SisDacOffset0") == 0)
	    return secondsSince1996 - SIS_DAC_OFFSET_0_SAMPLE_PERIOD + 1;
	else if (strcmp(structName, "L1SisDacOffset1") == 0)
	    return secondsSince1996 - SIS_DAC_OFFSET_1_SAMPLE_PERIOD + 1;
	else if (strcmp(structName, "L1SisDiagnosticEvent") == 0) return
		getStartForStructWithStopTime(secondsSince1996, structName);
	else if (strcmp(structName, "L1SisEventCycle") == 0)
	    return secondsSince1996 - SIS_EVENT_CYCLE_SAMPLE_PERIOD + 1;
	else if (strcmp(structName, "L1SisHighPriorityRate") == 0)
	    return secondsSince1996 - SIS_HIGH_PRIORITY_RATE_SAMPLE_PERIOD + 1;
	else if (strcmp(structName, "L1SisHskp") == 0)
	    return secondsSince1996 - SIS_HOUSEKEEPING_SAMPLE_PERIOD + 1;
	else if (strcmp(structName, "L1SisLowPriorityRate") == 0)
	    return secondsSince1996 - SIS_LOW_PRIORITY_RATE_SAMPLE_PERIOD + 1;
	else if (strcmp(structName, "L1SisSubset") == 0)
	    return secondsSince1996 - SIS_SUBSET_SAMPLE_PERIOD + 1;
	else if (strcmp(structName, "L1SisSummary") == 0)
	    return secondsSince1996 - SIS_SUMMARY_SAMPLE_PERIOD + 1;
    }

	/* if we got here, did not recognize the structure name */
    fprintf(stderr, "getWorkingStartTime: ERROR\n");
    fprintf(stderr, "  Struct name ->%s<- not recognized.  Exiting.\n\n",
								structName);
    exit(0);
}

    /* in explanation of the strangely named subroutine, structs which */
    /* are acyclic and have stop times associated with their intervals */
    /* of validity instead of a constant period, need to be treated in */
    /* a special manner to see if secondsSince1996 falls within the */
    /* valid interval of the structure previous to the request time, */
    /* secondsSince1996... */

uint32 getStartForStructWithStopTime(uint32 secondsSince1996, char *structName){
    uint32 startAndStopOfPreviousStruct[2];
    uint32 previousStart, previousStop;
    char path[256];
    struct tm *pTimeStruct;
    char instrumentID;
    char suffix[10];

    /* printf("getStartForStructWithStopTime: in\n"); */
    pTimeStruct = getDateFromSecondsSince1996(secondsSince1996);
    instrumentID = getInstrumentIDFromStructName(structName);
    getDataFileSuffixFromStructName(suffix, structName);
    generateDataFilePath(path, pTimeStruct, instrumentID, suffix);

    getBeginEndSecsOfStructPriorTo(startAndStopOfPreviousStruct,
					secondsSince1996, path, structName);
    previousStart = startAndStopOfPreviousStruct[0];
    previousStop = startAndStopOfPreviousStruct[1];

    /* printf("previousStart = %ld\n", previousStart); */
    /* printf("previousStop = %ld\n", previousStop); */

	/* if getBeginEndSecsOfStructPriorTo returned zeros, it means no */
	/* previous non-zero struct was found.  stick with looking for */
	/* start tags greater than secondsSince1996. */
    if (previousStop == 0) return secondsSince1996;

	/* else if secondsSince1996 occurs while the previous struct was */
	/* valid, return the previous structs start time as the "working */
	/* start time." */
    else if (secondsSince1996 <= previousStop) return previousStart;

	/* else previous struct was not valid at secondsSince1996, so */
	/* again, stick with looking for start tags greater than */
	/* secondsSince1996. */
    else return secondsSince1996;
}
